  #****s* SecDec/loop/perlsrc/preparenumerics.pl
  #  NAME
  #    preparenumerics.pl
  #
  #  USAGE
  #  is called from polenumerics.pl
  # 
  #  USES 
  #  $paramfile, header.pm, writefiles.pm, dirform.pm
  #  $polestructure determined by polenumerics.pl,
  #
  #  USED BY 
  #    
  #  PURPOSE
  #  writes the fortran files sf*.f, intfile*.f, and the files 
  #  makefile*, subfile.pl, for each given
  #  polestructure, order, number of variables 
  #  in the corresponding subdirectory of $currentdir.
  #  also writes $jobfile to submit the jobs to a queue in batch mode
  #    
  #  INPUTS
  #  $paramfile (default is param.input) read via module header
  #  parameters parsed via ARGV:
  #  jj: eps^jj
  #  numvar: number of integration variables, or Nn if IBP not used
  #  maxpole: maximal pole for this structure
  #  polestruct: [i]l[j]h[h] or sec*P[i]l[j]h[h], i logarithmic poles, j linear poles, h higher poles
  #    
  #  RESULT
  #  new functions in $currentdir and subdirectories corresponding to polestructure
  #    
  #  SEE ALSO
  #  polenumerics.pl, subexp.pl writefiles.pm
  #   
  #****
  #
use Getopt::Long;
GetOptions("parameter=s" => \$paramfile, "template=s"=>\$templatefile, "dirwork=s"=>\$workingdir);
unless ($paramfile) {
  $paramfile = "param.input";
}
unless ($templatefile) {
  $templatefile = "Template.m";
}
$wdstring="";
if($workingdir){
 $workingdir=~s/\/$//;
 $wdstring="-d=$workingdir ";
 $templatefile = "$workingdir/$templatefile";
 $paramfile = "$workingdir/$paramfile"
}

use lib "perlsrc";
use header;
use writefiles;
use dirform;
my %hash_var=header::readparams($paramfile);
$dirbase=`pwd`;
chomp $dirbase;
my $language=$hash_var{"language"};
unless ($language) {$language="fortran"};
if ($language eq "fortran"){$suf=".f"}else{$suf=".c"};
$compiler=$hash_var{"compiler"};
unless ($compiler) {$compiler="gfortran"};
$subdir=$hash_var{"diry"};
$diry=dirform::norm("${dirbase}/$subdir");
$currentdir=$hash_var{"currentdir"};
$graph=$hash_var{"graph"};
$externalegs=$hash_var{"externalegs"};

unless ($currentdir) {
if($workingdir){
if ($workingdir=~m/^\//){
  $diry=dirform::norm("$workingdir/$subdir");
  $currentdir="$diry/$graph";
 } else {
  $subdir="$workingdir/$subdir";
  $diry=dirform::norm("${dirbase}/$subdir");
  $currentdir="$diry/$graph"
 }
} else {
 $currentdir="$diry/$graph"
}
}
$basespath=$hash_var{"basespath"};
unless ($basespath) {$basespath="$dirbase/basesv5.1"};
$basespath=~s/\/loop\//\//;
$basespath=~s/\/general\//\//;
$Nn=$hash_var{"Nn"};
$cubapath=$hash_var{"cubapath"};
unless ($cubapath) {$cubapath="$dirbase/Cuba-2.1"};
$cubapath=~s/\/loop\//\//;
$cubapath=~s/\/general\//\//;
$routine=$hash_var{"integrator"};
unless($routine){$routine=0};
if($routine){$integpath=$cubapath}else{$integpath=$basespath};
$point=$hash_var{"pointname"};
unless ($point) {$point="DEFAULT"};
$processlimit=$hash_var{"processlimit"};
unless($processlimit){$processlimit=200};
if($processlimit<20){$processlimit=20};
$exe=$hash_var{"exe"};
if ($exe ne "0") {unless ($exe) {$exe=4};};
$clusterflag=$hash_var{"clusterflag"};
$local=0;
if($clusterflag==0){if($exe>0){$local=1}};
$cputime=$hash_var{"cputime"};
unless($cputime){$cputime=1000};
#
################## end of input parameters ################
#
$jj=$ARGV[0];
$jj=~s/m/-/;
$numvar=$ARGV[1];
$maxpole=$ARGV[2];
$maxpole=~s/m/-/;
$polestruct=$ARGV[3];
my $maindir = "$currentdir/$polestruct";
if ($numvar<$Nn) {
$currentdir="$maindir/epstothe$jj/$numvar";
} else {
$currentdir="$maindir/epstothe$jj";
$numvar=$Nn-1
}
$maxsize=$hash_var{"grouping"};
if($maxsize ne "0"){unless($maxsize){$maxsize=2000000}};
$funcount=1;
$fmax[0]=0;
$groupcount=0;
$sizesum=0;
while (-e "$currentdir/f$funcount$suf") {
 $filesize = -s "$currentdir/f$funcount$suf";
 $sizesum=$sizesum+$filesize;
 if($sizesum>$maxsize){
  $groupcount++;
  $fmax[$groupcount]=$funcount;
  $sizesum=0
 }
 $funcount++
}
$funcount--;
unless ($sizesum==0){
 $groupcount++;
 $fmax[$groupcount]=$funcount
}
$filepoint=$point;
$filepoint=~s/DEFAULT//;
$batchsystem=$hash_var{"batch"};
$integpathstring="";
if($language eq "fortran"){
 for ($kk=1;$kk<=$groupcount;$kk++) {
	$minf=$fmax[$kk-1]+1;
	$maxf=$fmax[$kk];
	if($local==0){$integpathstring="$graph/$polestruct"}else{$integpathstring=$compiler};
	@makeargs=("$currentdir/${filepoint}make${kk}file","$integpath/$integpathstring",$compiler,$point,$kk,$minf,$maxf,$routine);
	writefiles::makemake(@makeargs);#creates makefiles
	$singflag=$minf;
	$prestring="P$polestruct";
	if ($ARGV[1]<$Nn) {$prestring="${prestring}N$numvar"};
	$sumname="G$filepoint${prestring}sf$kk";
	if($minf<$maxf){
	 $singflag=0;
	 @sumargs=("$currentdir/${filepoint}sf$kk$suf",$sumname,$minf,$maxf,$prestring);
	 writefiles::makesum(@sumargs); #creates sf*$suf
	}
	if ($singflag==0){
	 $integrand=$sumname;
	} else {
	 $integrand="${prestring}f$singflag";
	}
	@intargs=("$currentdir/${filepoint}intfile$kk$suf",$jj,$kk,$numvar,$maxpole,$paramfile,$integrand);
	writefiles::makeint(@intargs); #creates intfile*$suf resp intfile*.c
	$maxpole=~s/m/-/;
	if ($clusterflag==1) {
	 if ($ARGV[1]<$Nn) {
	  if ($filepoint ne "") {
	   $jobfilename="$currentdir/$polestruct.${point}.$jj.$numvar.$kk"
	  } else {
	   $jobfilename="$currentdir/$polestruct.$jj.$numvar.$kk"
	  }
	 } else {
	  if ($filepoint ne "") {
	   $jobfilename="$currentdir/$polestruct.${point}.$jj.$kk"
	  } else {
	   $jobfilename="$currentdir/$polestruct.$jj.$kk"
	  }
	 }
	@jobargs=($batchsystem,$jobfilename,$currentdir,"${filepoint}intfile$kk.exe",$cputime,1);
	writefiles::makejob(@jobargs); #creates job submission files
	}
 }#next kk
}else{
 $Ccompiler="g++";###############################################################
 for ($kk=1;$kk<=$groupcount;$kk++) {
	$minf=$fmax[$kk-1]+1;
	$maxf=$fmax[$kk];
	if($local==0){$integpathstring="$graph/$polestruct"}else{$integpathstring=$compiler};
	@makeargs=("$currentdir/${filepoint}make${kk}file","$integpath/$integpathstring",$Ccompiler,$point,$kk,$minf,$maxf);
	writefiles::makemakeC(@makeargs);
	$prestring="P$polestruct";
	if ($ARGV[1]<$Nn) {$prestring="${prestring}N$numvar"};
	@intargs=("$currentdir/${filepoint}intfile$kk$suf",$minf,$maxf,$prestring,
	 $jj,$kk,$numvar,$maxpole,$paramfile);
	writefiles::makeintC(@intargs);
	if ($clusterflag==1) {
	 if ($ARGV[1]<$Nn) {
	  if ($filepoint ne "") {
	   $jobfilename="$currentdir/$polestruct.${point}.$jj.$numvar.$kk"
	  } else {
	   $jobfilename="$currentdir/$polestruct.$jj.$numvar.$kk"
	  }
	 } else {
	  if ($filepoint ne "") {
	   $jobfilename="$currentdir/$polestruct.${point}.$jj.$kk"
	  } else {
	   $jobfilename="$currentdir/$polestruct.$jj.$kk"
	  }
	 }
	@jobargs=($batchsystem,$jobfilename,$currentdir,"${filepoint}intfile$kk.exe",$cputime,1);
	writefiles::makejob(@jobargs); #creates job submission files
	}
 }#next kk
} 

@subargs=("$currentdir/${filepoint}subfile.pl",$jj,$Nn,$point,$graph,$polestruct,$processlimit,$ARGV[1],$exe,
 "$dirbase/perlsrc",$batchsystem,$currentdir,$clusterflag,$language);
writefiles::makesub(@subargs); #creates subfile.pl
@makemakerunargs=("$currentdir/${filepoint}make.pl",$jj,$Nn,$point,$graph,$polestruct,$language);
writefiles::makemakerun(@makemakerunargs); #creates make.pl
open (INFO,">$currentdir/${filepoint}info");
 print INFO "Number of integrations = $groupcount";
close INFO;





